@@ -11,6 +11,8 @@ use syn::{
 #[doc(hidden)]
 #[derive(Debug, Default, FromMeta)]
 pub struct Attrs {
+    #[darling(default)]
+    new: bool,
     #[darling(default)]
     copy: bool,
     #[darling(default)]
@@ -20,6 +22,8 @@ pub struct Attrs {
     #[darling(default)]
     sqlx: bool,
     #[darling(default)]
+    async_graphql: bool,
+    #[darling(default)]
     borrow: Option<syn::Path>,
     #[darling(default)]
     try_from: Option<syn::LitStr>,
@@ -83,6 +87,14 @@ fn do_newtype(mut attrs: Attrs, item: Item) -> Result<TokenStream, syn::Error> {
         }
     };
 
+    let async_graphql = if attrs.async_graphql {
+        Some(quote! {
+            async_graphql::scalar!(#ident);
+        })
+    } else {
+        None
+    };
+
     let deref = if attrs.opaque {
         None
     } else {
@@ -104,13 +116,40 @@ fn do_newtype(mut attrs: Attrs, item: Item) -> Result<TokenStream, syn::Error> {
         })
     };
 
+    let new = if attrs.new {
+        Some(quote! {
+            impl #ident {
+                pub fn new(input: #ty) -> Self {
+                    Self(input)
+                }
+            }
+
+            impl From<#ty> for #ident {
+                fn from(x: #ty) -> Self {
+                    Self(x)
+                }
+            }
+        })
+    } else {
+        None
+    };
+
+    let trait_impl = quote! {
+        impl ::nova::NewType for #ident {
+            type Inner = #ty;
+        }
+    };
+
     let out = quote! {
         #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, core::hash::Hash)]
         #copy
         #serde
         #sqlx
         #visibility struct #ident(#ty);
+        #async_graphql
         #deref
+        #new
+        #trait_impl
     };
 
     Ok(out)
